//////////////////////////////////////////////////////////////////////////
// Copyright 2005-2008 Aspose Pty Ltd. All Rights Reserved.
//
// This file is part of Aspose.Editor. The source code in this file
// is only intended as a supplement to the documentation, and is provided
// "as is", without warranty of any kind, either expressed or implied.
//////////////////////////////////////////////////////////////////////////

using System;
using System.Collections;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Windows.Forms;
#if ASPOSE_EDITOR_DESKTOP
namespace Aspose.Editor.Desktop
#elif ASPOSE_EDITOR_CLIENT
namespace Aspose.Editor.Client
#endif
{
    /// <summary>
    /// Provides integration of the KeyotiRapidSpell for .NET with Aspose.Editor control.
    /// </summary>
    /// <example>
    /// In order to enable spell checking functionality in the Aspose.Editor it is required
    /// to add single line of code right after the Aspose.Editor instance is created.
    /// <code>
    /// // Editor conrol instance is created.
    /// Editor = new EditorControl();
    /// // Adapter is installed. The line below is the only line required. Make sure this is not called twice.
    /// AsposeSpellAdapter.Install(Editor);
    /// </code>
    /// </example>
    [PluginClass("Aspose.Spell integration.")]
    public class AsposeSpellAdapter
    {
        /// <summary>
        /// Installs the adapter within the specified editor control. This method shall be called once
        /// for each editor control.
        /// </summary>
        /// <remarks>If this method is called multiple times for the same control then results are
        /// unpredictable.</remarks>
        public static void Install(EditorControl editor)
        {
            // Here we create instance of the adapter. It will create commands in the editor and thus
            // until editor is released adapter stays alive. We dont have to keep reference onto adapter.
            new AsposeSpellAdapter(editor);
        }


        /// <summary>
        /// Initializes new instance of this class.
        /// </summary>
        /// <param name="editor">Editor control.</param>
        public AsposeSpellAdapter(EditorControl editor)
        {
            mEditor = editor;
            mSpellDialog = new AsposeSpellDialog();
            // mSpellDialog.Parent = mEditor;
            // mSpellDialog.TopMost = true;

            mEditor.DocumentChanged += new EventHandler(Editor_DocumentChanged);

            SetupCommand();
            SetupToolbar();
            SetupMenu();
        }


        /// <summary>
        /// This method is called when new document is opened in the editor.
        /// </summary>
        private void Editor_DocumentChanged(object sender, EventArgs e)
        {

        }


        /// <summary>
        /// Opens the stream for reading of embedded resource data.
        /// </summary>
        /// <param name="resource">Name of resource.</param>
        /// <returns>Stream.</returns>
        private Stream GetResourceStream(string resource)
        {
            Assembly assembly = Assembly.GetExecutingAssembly();
            string resName = string.Format("{0}.{1}", assembly.GetName().Name, resource);
            return assembly.GetManifestResourceStream(resName);
        }


        /// <summary>
        /// Creates command for the spell dialog.
        /// </summary>
        private void SetupCommand()
        {
            Image commandIcon = null;

            using ( Stream stream = GetResourceStream("SpellDialog.gif") )
                if ( stream != null )
                    commandIcon = Image.FromStream(stream);

            mEditor.Commands.Add(SpellCommandName, null, "Check spelling of the current document now.",
                Keys.F7, new CommandHandler(CheckSpelling), commandIcon, new ConditionHandler(CanCheckSpelling));
        }


        /// <summary>
        /// Creates tool bar buttons for spell command.
        /// </summary>
        private void SetupToolbar()
        {
            ArrayList toolBars = new ArrayList(mEditor.Bars.Tools);
            toolBars.Add(new StandardToolBar(mEditor, SpellCommandName));
            mEditor.Bars.Tools = toolBars.ToArray(typeof (Control)) as Control[];
        }


        /// <summary>
        /// Creates menu items for spell commands.
        /// </summary>
        private void SetupMenu()
        {
            const string ToolsMenuName = "&Tools";
            StandardMenuBar menuBar = mEditor.Bars.Menu as StandardMenuBar;

            // Create list of menu items for spell commands.
            MenuItem[] spellItems = null;
            spellItems = new MenuItem[] { mEditor.Commands[SpellCommandName].GetMenuItem(), };
            Debug.Assert(spellItems != null);

            // Find "Tools" and "Help" main menu item.
            MenuItem toolItem = null;
            int helpItemIndex = 0;
            for ( int i = 0; i < menuBar.ItemCount; i++ )
            {
                MenuItem item = menuBar.GetItemAt(i);
                string itemText = item.Text.Replace("&", string.Empty);
                if ( itemText == ToolsMenuName )
                    toolItem = item;
                if ( itemText == "Help" )
                    helpItemIndex = i;
            }

            if ( toolItem != null )
            // If tools found then we put spell items under it.
                toolItem.MenuItems.AddRange(spellItems);

            else
            if ( 0 <= helpItemIndex )
            // If tool is not found but help is found then we create tools menu before help.
                menuBar.InsertItem(new MenuItem(ToolsMenuName, spellItems), helpItemIndex);

            else
            // If neither tools nor help is found then we append tools menu.
                menuBar.AddItem(new MenuItem(ToolsMenuName, spellItems));
        }


        /// <summary>
        /// This method is called when check spelling command is invoked.
        /// </summary>
        private void CheckSpelling(EditorControl editorControl)
        {
            mSpellDialog.CheckSpelling(mEditor);
        }


        /// <summary>
        /// This method is called when <see cref="Control.Enabled"/> property of the UI item
        /// which corresponds to the spell commands is set.
        /// </summary>
        private bool CanCheckSpelling(EditorControl editorControl)
        {
            Debug.Assert(null != editorControl);

            // When editor is in ReadOnly mode we must prevent any commands which can change
            // document content from executing.
            return (editorControl.Document != null && !editorControl.Options.IsReadOnly);
        }


        /// <summary>
        /// Editor this adapter works with.
        /// </summary>
        private EditorControl mEditor;

        /// <summary>
        /// Spell dialog.
        /// </summary>
        private AsposeSpellDialog mSpellDialog;

        /// <summary>
        /// Name of the command used to check the document for spell errors.
        /// This value matches menu item name for this command.
        /// </summary>
        private const string SpellCommandName = "Check &Spelling";
    }
}
