//////////////////////////////////////////////////////////////////////////
// Copyright 2005-2008 Aspose Pty Ltd. All Rights Reserved.
//
// This file is part of Aspose.Editor. The source code in this file
// is only intended as a supplement to the documentation, and is provided
// "as is", without warranty of any kind, either expressed or implied.
//////////////////////////////////////////////////////////////////////////

using System;
using System.Collections;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Reflection;
using System.Windows.Forms;
using Keyoti.RapidSpell;

#if ASPOSE_EDITOR_DESKTOP
namespace Aspose.Editor.Desktop
#elif ASPOSE_EDITOR_CLIENT
namespace Aspose.Editor.Client
#endif
{
    /// <summary>
    /// Implements KeyotiRapidSpell for .NET adapter used to integrate spell checking
    /// functionality into Aspose.Editor.
    /// </summary>
    internal class KeyotiRapidSpellAdapterImpl : ISpellCheckableTextComponent, IAYTTextBox
    {
        /// <summary>
        /// Creates instance of the adapter and attaches it to the specified editor.
        /// </summary>
        /// <param name="editorControl">The editor control this spell checker shall work with.</param>
        /// <param name="isInstallSpellDialog">If set to <b>true</b> then installs spell dialog feature
        /// into the editor.</param>
        /// <param name="isInstallSpellAsYouType">If set to <b>true</b> then installs check spell as you
        /// type feature into the editor.</param>
        /// <param name="isEnableSpellAsYouType">If set to <b>true</b> then check spelling as you type
        /// feature is enabled by default, <b>false</b> otherwise. In both cases when editor control is
        /// running user will be able to enable and disable this feature manually.</param>
        public KeyotiRapidSpellAdapterImpl(EditorControl editorControl, bool isInstallSpellDialog,
            bool isInstallSpellAsYouType, bool isEnableSpellAsYouType)
        {
            if (!isInstallSpellDialog && !isInstallSpellAsYouType)
                return;

            mEditor = editorControl;

            if (isInstallSpellDialog)
                InitSpellDialog();

            if (isInstallSpellAsYouType)
            {
                Editor.Idle += new EventHandler(OnIdle);
                Editor.ContentChanged += new ContentChangedEventHandler(OnContentChanged);
                Editor.AttributesChanged += new AttributesChangedEventHandler(OnAttributesChanged);
                mIsEnableSpellAsYouType = isEnableSpellAsYouType;
                InitSpellAutomatic();
            }

            if (isInstallSpellDialog && isInstallSpellAsYouType)
            {
                InitSpellCoupler();
            }

            if (isInstallSpellDialog || isInstallSpellAsYouType)
            {
                Editor.DocumentChanged += new EventHandler(OnDocumentChanged);
                InitCommands_Toolbars();
                InitCommands_Menus();

                // When spell checker is installed there could be a document already opened in the editor.
                // We'll report that document has changed to inform spell checker of the existing content.
                if (Editor.Document != null)
                    OnDocumentChanged(this, EventArgs.Empty);
            }
        }


        /// <summary>
        /// Releases all references adapter has to the editor and events. After this method is
        /// called adapter cannot be used any longer. This method shall be called right before
        /// editor control is released (goes out of scope).
        /// </summary>
        public void Release()
        {
            if (mEditor != null)
            {
                mEditor.Idle -= new EventHandler(OnIdle);
                mEditor.ContentChanged -= new ContentChangedEventHandler(OnContentChanged);
                mEditor.AttributesChanged -= new AttributesChangedEventHandler(OnAttributesChanged);
                mEditor.DocumentChanged -= new EventHandler(OnDocumentChanged);
                SetupEvents(false);
                mEditor = null;
            }
        }


        /// <summary>
        /// Name of the command used to check the document for spell errors.
        /// This value matches menu item name for this command.
        /// </summary>
        public const string SpellDialogCommandName = "Check &Spelling";

        /// <summary>
        /// Name of the command used to toggle state of the check spelling as you type feature.
        /// This value matches menu item name for this command.
        /// </summary>
        public const string SpellAsYouTypeCommandName = "Check Spelling &Automatically";

        #region Implementation: InitSpellDialog, InitSpellAutomatic, InitSpellCoupler, InitCommands

        /// <summary>
        /// Opens the stream for reading of embedded resource data.
        /// </summary>
        /// <param name="resource">Name of resource.</param>
        /// <returns>The stream reference.</returns>
        private Stream GetResourceStream(string resource)
        {
            Assembly assembly = Assembly.GetExecutingAssembly();
            string resName = string.Format("{0}.{1}", "Aspose.Editor.Desktop", resource);
            return assembly.GetManifestResourceStream(resName);
        }


        /// <summary>
        /// Creates command for the spell checking dialog feature.
        /// </summary>
        private void InitSpellDialog()
        {
            Image commandIcon = null;

            using (Stream stream = GetResourceStream("RapidSpellDialog.gif"))
                if (stream != null)
                    commandIcon = Image.FromStream(stream);

            Editor.Commands.Add(SpellDialogCommandName, null, "Check spelling of the current document now.",
                Keys.F7, new CommandHandler(CheckSpelling), commandIcon, new ConditionHandler(canCheckSpelling));

            mRapidSpellDialog = new RapidSpellDialog();
            mRapidSpellDialog.TopMost = true;
            mRapidSpellDialog.Parent = Editor;
            mRapidSpellDialog.ThirdPartyTextComponentToCheck = this;
        }


        /// <summary>
        /// Creates command for the check spelling as you type feature.
        /// </summary>
        private void InitSpellAutomatic()
        {
            Image commandIcon = null;
            using (Stream stream = GetResourceStream("RapidSpellAsYouType.gif"))
                if (stream != null)
                    commandIcon = Image.FromStream(stream);

            Editor.Commands.Add(SpellAsYouTypeCommandName, null, "Enables or disables automatic spell checking as text is typed.",
                Keys.None, new CommandHandler(ToggleCheckSpelling), commandIcon, new ConditionHandler(canCheckSpelling), new ConditionHandler(isPushedCheckSpelling));

            mRapidSpellAsYouType = new RapidSpellAsYouType();

            // We dont use context menus yet, so disable them.
            mRapidSpellAsYouType.ShowSuggestionsContextMenu = true;//Keyoti Change

            // In .NET 2.0 another rapidspell dll is used which has slightly different interface.
            // We disable context menu strip on this interface.
#if NET20
            mRapidSpellAsYouType.ContextMenuStripEnabled = true;//Keyoti Change
#endif
            mRapidSpellAsYouType.CheckAsYouType = mIsEnableSpellAsYouType;

            mRapidSpellAsYouType.TextComponent = this;
        }


        /// <summary>
        /// Creates coupler which is required only if both spell dialog and spell
        /// as you type features are installed.
        /// </summary>
        private void InitSpellCoupler()
        {
            mRapidSpellAYTDialogCoupler = new RapidSpellAYTDialogCoupler();
            mRapidSpellAYTDialogCoupler.RapidSpellDialog = mRapidSpellDialog;
            mRapidSpellAYTDialogCoupler.RapidSpellAsYouType = mRapidSpellAsYouType;
        }


        /// <summary>
        /// Creates tool bar buttons for spell commands.
        /// </summary>
        private void InitCommands_Toolbars()
        {
            ArrayList toolBars = new ArrayList(Editor.Bars.Tools);
            if (mRapidSpellDialog != null && mRapidSpellAsYouType != null)
                toolBars.Add(new StandardToolBar(Editor, SpellDialogCommandName, SpellAsYouTypeCommandName));

            else
                if (mRapidSpellDialog != null)
                    toolBars.Add(new StandardToolBar(Editor, SpellDialogCommandName));

                else
                    if (mRapidSpellDialog != null)
                        toolBars.Add(new StandardToolBar(Editor, SpellAsYouTypeCommandName));

            Editor.Bars.Tools = toolBars.ToArray(typeof(Control)) as Control[];
        }


        /// <summary>
        /// Creates menu items for spell commands.
        /// </summary>
        private void InitCommands_Menus()
        {
            const string ToolsMenuName = "&Tools";
            StandardMenuBar menuBar = Editor.Bars.Menu as StandardMenuBar;

            // Create list of menu items for spell commands.
            MenuItem[] spellItems = null;

            if (mRapidSpellDialog != null && mRapidSpellAsYouType != null)
                spellItems = new MenuItem[] { Editor.Commands[SpellDialogCommandName].GetMenuItem(), Editor.Commands[SpellAsYouTypeCommandName].GetMenuItem(), };

            else
                if (mRapidSpellDialog != null)
                    spellItems = new MenuItem[] { Editor.Commands[SpellDialogCommandName].GetMenuItem(), };

                else
                    if (mRapidSpellAsYouType != null)
                        spellItems = new MenuItem[] { Editor.Commands[SpellAsYouTypeCommandName].GetMenuItem(), };

            Debug.Assert(spellItems != null);

            // Find "Tools" and "Help" main menu item.
            MenuItem toolItem = null;
            int helpItemIndex = 0;
            for (int i = 0; i < menuBar.ItemCount; i++)
            {
                MenuItem item = menuBar.GetItemAt(i);
                string itemText = item.Text.Replace("&", string.Empty);
                if (itemText == ToolsMenuName)
                    toolItem = item;
                if (itemText == "Help")
                    helpItemIndex = i;
            }

            if (toolItem != null)
                // If tools found then we put spell items under it.
                toolItem.MenuItems.AddRange(spellItems);

            else
                if (0 <= helpItemIndex)
                    // If tool is not found but help is found then we create tools menu before help.
                    menuBar.InsertItem(new MenuItem(ToolsMenuName, spellItems), helpItemIndex);

                else
                    // If neither tools nor help is found then we append tools menu.
                    menuBar.AddItem(new MenuItem(ToolsMenuName, spellItems));
        }


        /// <summary>
        /// This method is called when <see cref="ToolBarButton.Pushed"/> property of the UI item
        /// which corresponds to the check spelling as you type command is set.
        /// </summary>
        private bool isPushedCheckSpelling(EditorControl editor)
        {
            return mRapidSpellAsYouType.CheckAsYouType;
        }


        /// <summary>
        /// This method is called when check spelling as you type command is invoked.
        /// </summary>
        private void ToggleCheckSpelling(EditorControl editor)
        {
            mRapidSpellAsYouType.CheckAsYouType = !mRapidSpellAsYouType.CheckAsYouType;
            if (Editor.ActiveView != null)
                Editor.ActiveView.Refresh();
        }


        /// <summary>
        /// This method is called when check spelling command is invoked.
        /// </summary>
        private void CheckSpelling(EditorControl editorControl)
        {
            mRapidSpellDialog.Check();
        }


        /// <summary>
        /// This method is called when <see cref="Control.Enabled"/> property of the UI item
        /// which corresponds to the spell commands is set.
        /// </summary>
        private bool canCheckSpelling(EditorControl editorControl)
        {
            // When editor is in ReadOnly mode we must prevent any commands which can change
            // document content from executing.
            return (editorControl.Document != null && !editorControl.Options.IsReadOnly);
        }


        private RapidSpellDialog mRapidSpellDialog;
        private RapidSpellAsYouType mRapidSpellAsYouType;
        private RapidSpellAYTDialogCoupler mRapidSpellAYTDialogCoupler;

        #endregion

        #region Implementation: ISpellCheckableTextComponent

        /// <summary>
        /// Occurs when the editor control receives focus.
        /// </summary>
        public event EventHandler GotFocus;

        /// <summary>
        /// Scrolls the editor view to the current caret position.
        /// </summary>
        void ISpellCheckableTextComponent.ScrollToCaret()
        {
            Editor.Selection.ScrollIntoView();
        }


        /// <summary>
        /// Gets or sets a value indicating whether the selected text in the text box control remains
        /// highlighted when the control loses focus.
        /// </summary>
        /// <remarks>Editor doesn't hide selection when focus is lost. This cannot be changed.</remarks>
        bool ISpellCheckableTextComponent.HideSelection
        {
            get { return false; }
            set { }
        }


        /// <summary>
        /// Gets or sets the current text in the text box.
        /// </summary>
        string ISpellCheckableTextComponent.Text
        {
            get { return Text; }
            set { }
        }


        /// <summary>
        /// Gets or sets the starting point of text selected in the text box.
        /// </summary>
        int ISpellCheckableTextComponent.SelectionStart
        {
            get { return SelectionStart; }
            set { SelectionStart = value; }
        }


        /// <summary>
        /// Gets or sets the number of characters selected in the text box.
        /// </summary>
        int ISpellCheckableTextComponent.SelectionLength
        {
            get { return SelectionLength; }
            set { SelectionLength = value; }
        }


        /// <summary>
        /// Gets or sets selected text in the editor control.
        /// </summary>
        string ISpellCheckableTextComponent.SelectedText
        {
            get { return SelectedText; }
            set { SelectedText = value; }
        }


        #endregion

        #region Implementation: Events

        public event EventHandler Enter;

        public event EventHandler Leave;

        public event KeyEventHandler KeyUp;

        public event KeyEventHandler KeyDown;

        public event EventHandler Resize;

        public event EventHandler ControlPaint;

        public event EventHandler TextChanged;

        public event EventHandler FormatChanged;

        public event EventHandler ScrollCaret;

        public event EventHandler VScrolled;

        public event EventHandler VScrollEnd;

        public event EventHandler HScrolled;

        public event MouseEventHandler MouseDown;

        public event MouseEventHandler MouseUp;

        public event MouseEventHandler MouseWheel;

        event EventHandler IAYTTextBox.Disposed
        {
            add { Editor.Disposed += value; }
            remove { Editor.Disposed -= value; }
        }


        private void SetupEvents(bool isSet)
        {
            // Each time document changes we may have view changed as well. Thus we have to unwire
            // all events from the view and wire them back to a new view.
            if (mView != null)
            {
                if (mRapidSpellDialog != null)
                    mView.GotFocus -= new EventHandler(View_OnGotFocus);

                if (mRapidSpellAsYouType != null)
                {
                    //v4
                    mView.KeyPress -= new KeyPressEventHandler(mView_KeyPress);//for RSDT.NET v4
                    mView.Enter -= new EventHandler(View_OnEnter);
                    mView.Leave -= new EventHandler(View_OnLeave);
                    mView.Paint -= new PaintEventHandler(View_OnPaint);
                    mView.MouseDown -= new MouseEventHandler(mView_MouseDown);//Keyoti Change
                }

                mView = null;
            }

            if (isSet && Editor.ActiveView != null)
            {
                mView = Editor.ActiveView;

                if (mRapidSpellDialog != null)
                    mView.GotFocus += new EventHandler(View_OnGotFocus);

                if (mRapidSpellAsYouType != null)
                {
                    //v4
                    mView.KeyPress += new KeyPressEventHandler(mView_KeyPress);//for RSDT.NET v4
                    mView.Enter += new EventHandler(View_OnEnter);
                    mView.Leave += new EventHandler(View_OnLeave);
                    mView.Paint += new PaintEventHandler(View_OnPaint);
                    mView.MouseDown += new MouseEventHandler(mView_MouseDown);//Keyoti Change
                }

                // Since document has changed we need to notify spell checker of content change.
                if (TextChanged != null)
                    TextChanged(this, EventArgs.Empty);
            }
        }
        //v4
        void mView_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (KeyPress != null) KeyPress(this, e);
        }

        //Keyoti Change
        void mView_MouseDown(object sender, MouseEventArgs e)
        {
            if (MouseDown != null) MouseDown(this, e);
        }


        private DocumentView mView;

        /// <summary>
        /// This event is used with <see cref="mRapidSpellDialog"/>.
        /// </summary>
        private void View_OnGotFocus(object sender, EventArgs e)
        {
            if (GotFocus != null)
                GotFocus(this, EventArgs.Empty);
        }


        /// <summary>
        /// This event is used with <see cref="mRapidSpellAsYouType"/>.
        /// </summary>
        private void View_OnPaint(object sender, PaintEventArgs e)
        {
            if (ControlPaint != null)
            {
                GraphicsState graphicsState = e.Graphics.Save();

                e.Graphics.ResetTransform();
                e.Graphics.ResetClip();

                ControlPaint(this, e);

                if (mKeyotiTextBoxUtilityAdapter.BackgroundBMP != null)
                    e.Graphics.DrawImageUnscaled(mKeyotiTextBoxUtilityAdapter.BackgroundBMP, Point.Empty);

                e.Graphics.Restore(graphicsState);
            }
        }


        /// <summary>
        /// This event is used with <see cref="mRapidSpellAsYouType"/>.
        /// </summary>
        private void View_OnLeave(object sender, EventArgs e)
        {
            if (Leave != null)
                Leave(this, EventArgs.Empty);
        }


        /// <summary>
        /// This event is used with <see cref="mRapidSpellAsYouType"/>.
        /// </summary>
        private void View_OnEnter(object sender, EventArgs e)
        {
            if (Enter != null)
                Enter(this, EventArgs.Empty);
        }


        /// <summary>
        /// This event is used with <see cref="mRapidSpellDialog"/> and <see cref="mRapidSpellAsYouType"/>.
        /// </summary>
        private void OnDocumentChanged(object sender, EventArgs e)
        {
            SetupEvents(true);
        }


        /// <summary>
        /// This event is used with <see cref="mRapidSpellAsYouType"/>.
        /// </summary>
        private void OnIdle(object sender, EventArgs e)
        {
            // After document is closed we will receive OnIdle but we dont want to handle it.
            if (Editor.ActiveView == null)
                return;

            // This should update underlines collection.
            if (0 < mContentChangeCount && TextChanged != null)
            {
                Debug.WriteLine("TextChanged");
                mContentChangeCount--;
                TextChanged(this, EventArgs.Empty);
            }

            // This should reposition underlines.
            if (0 < mAttrChangeCount && FormatChanged != null)
            {
                Debug.WriteLine("AttrChanged");
                mAttrChangeCount--;
                FormatChanged(this, EventArgs.Empty);
            }

            // This should repaint underlines onto bitmap.
            RectangleF newWindow = Editor.ActiveView.Window;
            if (!mWindow.Equals(newWindow))
            {
                bool isHScroll = (mWindow.X != newWindow.X);
                bool isVScroll = (mWindow.Y != newWindow.Y);
                bool isZoom = !mWindow.Size.Equals(newWindow.Size);
                mWindow = newWindow;

                if (isZoom && Resize != null)
                {
                    Debug.WriteLine("Resize");
                    Resize(this, EventArgs.Empty);
                }

                else
                {
                    if (isHScroll && HScrolled != null)
                    {
                        Debug.WriteLine("HScrolled");
                        HScrolled(this, EventArgs.Empty);
                    }

                    if (isVScroll)
                    {
                        if (VScrolled != null)
                        {
                            Debug.WriteLine("VScrolled");
                            VScrolled(this, EventArgs.Empty);
                        }

                        if (VScrollEnd != null)
                            VScrollEnd(this, EventArgs.Empty);
                    }
                }
            }

            // Statements above update the view. Here we will refresh the view but since spell
            // check happens on a different thread underlines may not be available yet.
            Editor.ActiveView.Refresh();
        }


        /// <summary>
        /// This event is used with <see cref="mRapidSpellAsYouType"/>.
        /// </summary>
        private void OnAttributesChanged(object sender, AttributesChangedEventArgs e)
        {
            mAttrChangeCount++;
        }


        /// <summary>
        /// This event is used with <see cref="mRapidSpellAsYouType"/>.
        /// </summary>
        private void OnContentChanged(object sender, ContentChangedEventArgs e)
        {
            mContentChangeCount++;
        }


        private int mAttrChangeCount = 0;
        private int mContentChangeCount = 0;
        private RectangleF mWindow = RectangleF.Empty;

        #endregion

        #region Implementation: Layout query methods

        int IAYTTextBox.GetLineFromCharIndex(int index)
        {
            return 0;
        }


        int IAYTTextBox.GetBaselineOffsetAtCharIndex(int index)
        {
            if (Editor.ActiveView == null)
                return 0;

            RectangleF bounds;
            float glyphBaseline, lineHeight, lineBaseline;
            Editor.ActiveView.GetGlyphMeasurements(index, out bounds,
                out glyphBaseline, out lineBaseline, out lineHeight);

            if (glyphBaseline > lineBaseline)
                glyphBaseline = lineBaseline;

            RectangleF tempRect = new RectangleF(0, 0, 0, glyphBaseline);

            // We add 1 pixel because of possible rounding of points. Lower underlines look better.
            return Editor.ActiveView.PointToClient(tempRect).Height + 1;
        }


        int IAYTTextBox.GetCharIndexFromPos(Point pos)
        {
            if (Editor.ActiveView == null)
                return 0;

            RectangleF tempRect = Editor.ActiveView.ClientToPoint(new Rectangle(pos, Size.Empty));
            return Editor.ActiveView.GetGlyphOffset(tempRect.Location);
        }


        Point IAYTTextBox.GetPosFromCharIndex(int index)
        {
            RectangleF bounds;
            float glyphBaseline, unused, lineBaseline;
            Editor.ActiveView.GetGlyphMeasurements(index, out bounds,
                out glyphBaseline, out lineBaseline, out unused);

            // We can have clipped glyphs, in this case left top corner will move to the line boundary.
            if (glyphBaseline > lineBaseline)
                bounds.Y += lineBaseline - glyphBaseline;

            return Editor.ActiveView.PointToClient(bounds).Location;
        }


        int IAYTTextBox.GetFirstVisibleCharIndex()
        {
            return (Editor.ActiveView == null ? 0 :
                    Editor.ActiveView.GetGlyphOffset(Editor.ActiveView.Window.Location));
        }


        #endregion

        #region Implementation: Inherited from control

        IAsyncResult IAYTTextBox.BeginInvoke(Delegate d)
        {
            return Editor.BeginInvoke(d);
        }


        IAsyncResult IAYTTextBox.BeginInvoke(Delegate d, object[] parameters)
        {
            return Editor.BeginInvoke(d, parameters);
        }


        bool IAYTTextBox.IsDisposed
        {
            get { return Editor.IsDisposed; }
        }


        void IAYTTextBox.SuspendLayout()
        {
        }


        bool IAYTTextBox.IsHandleCreated
        {
            get
            {
                return (Editor.ActiveView == null ? false : Editor.ActiveView.IsHandleCreated);
            }
        }


        IntPtr IAYTTextBox.Handle
        {
            get
            {
                return (Editor.ActiveView == null ? IntPtr.Zero : Editor.ActiveView.Handle);
            }
        }


        Size IAYTTextBox.ClientSize
        {
            get
            {
                return (Editor.ActiveView == null ? Size.Empty : Editor.ActiveView.ClientSize);
            }
            set { }
        }


        void IAYTTextBox.Refresh()
        {
            // Since we fire all events in a row we dont want each event to refresh view but update it only.
            if (Editor.ActiveView != null)
                Editor.ActiveView.Update();
        }


        int IAYTTextBox.Width
        {
            get
            {
                return (Editor.ActiveView == null ? 0 : Editor.ActiveView.Width);
            }
            set { }
        }


        int IAYTTextBox.Height
        {
            get
            {
                return (Editor.ActiveView == null ? 0 : Editor.ActiveView.Height);
            }
            set { }
        }


        #endregion

        #region Implementation: Selection and text

        int IAYTTextBox.SelectionStart
        {
            get { return SelectionStart; }
            set { SelectionStart = value; }
        }


        int IAYTTextBox.SelectionLength
        {
            get { return SelectionLength; }
            set { SelectionLength = value; }
        }


        string IAYTTextBox.SelectedText
        {
            get { return SelectedText; }
            set { SelectedText = value; }
        }


        string IAYTTextBox.Text
        {
            get { return Text; }
            set { }
        }


        int IAYTTextBox.TextLength
        {
            get { return Text.Length; }
        }


        private string SelectedText
        {
            get
            {
                return (Editor.Document == null ? string.Empty : Editor.Selection.Range.Text);
            }
            set
            {
                if (Editor.Document != null)
                    Editor.Selection.Range.Text = value;
            }
        }


        private int SelectionStart
        {
            get
            {
                return (Editor.Document == null ? 0 : Editor.Selection.Range.Start);
            }
            set
            {
                if (Editor.Document != null)
                    Editor.Selection.Range.Start = value;
            }
        }


        private int SelectionLength
        {
            get
            {
                return (Editor.Document == null ? 0 : Editor.Selection.Range.Length);
            }
            set
            {
                if (Editor.Document != null)
                    Editor.Selection.Range.Length = value;
            }
        }


        private string Text
        {
            get
            {
                return (Editor.Document == null ? string.Empty : Editor.Document.GetRange().Text);
            }
        }


        #endregion

        #region Implementation: Context menu

        ContextMenu IAYTTextBox.DefaultContextMenu
        {
            get { return mDummyDefaultContextMenu; }
        }


        ContextMenu IAYTTextBox.ContextMenuDefault
        {
            get { return mDummyContextMenuDefault; }
        }


        ContextMenu IAYTTextBox.ContextMenu
        {
            /*
            get { return mDummyContextMenu; }
            set { mDummyContextMenu = value; }
            */
            //Keyoti Change
            get { return Editor.ContextMenu; }
            set { Editor.ContextMenu = value; }
        }


        private ContextMenu mDummyDefaultContextMenu = new ContextMenu();
        private ContextMenu mDummyContextMenuDefault = new ContextMenu();
        private ContextMenu mDummyContextMenu = new ContextMenu();

#if NET20
        ContextMenuStrip IAYTTextBox.ContextMenuStrip
        {
            /* get { return mContextMenuStrip; }
            set { mContextMenuStrip = value; }
             */
            //Keyoti Change
            get { return Editor.ContextMenuStrip; }
            set { Editor.ContextMenuStrip = value; }
        }


        ContextMenuStrip IAYTTextBox.ContextMenuStripDefault
        {
            get { return mContextMenuStripDefault; }
        }

        private ContextMenuStrip mContextMenuStrip = new ContextMenuStrip();
        private ContextMenuStrip mContextMenuStripDefault = new ContextMenuStrip();
#endif

        #endregion

        #region Implementation: Font, GetTBU
        /* v4; Comment if using with  RSDT.NET v3
        System.Drawing.Font IAYTTextBox.Font
        {
            get { return mDummyFont;  }
            set { mDummyFont = value; }
        }
*/

        TextBoxUtilityAdapter IAYTTextBox.GetTBU()
        {
            if (mKeyotiTextBoxUtilityAdapter == null)
                mKeyotiTextBoxUtilityAdapter = new KeyotiTextBoxUtilityAdapter(this);
            return mKeyotiTextBoxUtilityAdapter;
        }


        System.Drawing.Font mDummyFont;
        KeyotiTextBoxUtilityAdapter mKeyotiTextBoxUtilityAdapter;

        #endregion

        /// <summary>
        /// Returns view control of the editor this adapter is working with. Keyoti assumes that event
        /// sender is adapter but in fact it is not. Event sender is view control. We'll report that
        /// event sender is adapter but when spell checker code requires access to control it will
        /// implicitly convert the reference using this operator.
        /// </summary>
        public static implicit operator Control(KeyotiRapidSpellAdapterImpl adapterImpl)
        {
            return (adapterImpl.Editor.ActiveView == null ? null : adapterImpl.Editor.ActiveView);
        }


        internal EditorControl Editor
        {
            get
            {
                if (mEditor == null)
                    throw new InvalidOperationException("Editor is null.");

                return mEditor;
            }
        }


        private EditorControl mEditor;
        private bool mIsEnableSpellAsYouType;


        // For RSDT.NET v4
        public void Dispose()
        {
            mEditor.Dispose();
        }
        public bool ReadOnly
        {
            get { return false; }
        }

        public bool Enabled { get { return mEditor.Enabled; } }
        public event EventHandler Disposed;
        public event KeyPressEventHandler KeyPress;
        //----------------------------------------------------------
    }
}
