//////////////////////////////////////////////////////////////////////////
// Copyright 2005-2008 Aspose Pty Ltd. All Rights Reserved.
//
// This file is part of Aspose.Editor. The source code in this file
// is only intended as a supplement to the documentation, and is provided
// "as is", without warranty of any kind, either expressed or implied.
//////////////////////////////////////////////////////////////////////////

using System;
using System.IO;
using System.Net;
using System.Text;
using System.Drawing;
using System.Reflection;
using System.Diagnostics;
using System.Windows.Forms;
using Aspose.Editor.Client;
using MenuItem=Aspose.Editor.Client.MenuItem;

namespace Aspose.Editor.Demo
{
    /// <summary>
    /// Extends <see cref="EditorControl"/> with additional commands 'send' and 'receive'. This class
    /// implements 'live' ASP.NET demo for the editor control which is hosted on the aspose site.
    /// </summary>
    [PluginClass("Web Demo Application Plugin", "Provides document send, receive and upload capabilities for the editor.")]
    public class AppPlugin
    {
        /// <summary>
        /// This is an entry point for the plugin. It is invoked right before the editor is about
        /// to be displayed. Here user interface case be customized.
        /// </summary>
        public AppPlugin(EditorControl editor)
        {
            mEditor = editor;
            Debug.WriteLine("Plugin started.");

            // Uncomment to speed up rendering of selection.
            // mEditor.Options.IsTransparentSelection = true;

            // Uncomment to disable scroll limits.
            // mEditor.Options.IsKeepPageInsideWindow = false;

            ModifyNewCommand();
            InitAboutCommand();
            InitSaveAsCommand();
            InitSaveCommand();
            InitOpenCommand();
        }


        #region About Command

        /// <summary>
        /// Name of the "About Demo" command.
        /// </summary>
        private string CommandAboutDemoName = "App.About";

        /// <summary>
        /// Initializes command "About Demo"
        /// </summary>
        private void InitAboutCommand()
        {
            Command command = mEditor.Commands.Add(CommandAboutDemoName,
                "About Demo", "Displays about dialog for the web demo plugin.",
                new CommandHandler(CommandAboutDemo));

            StandardMenuBar menuBar = mEditor.Bars.Menu as StandardMenuBar;
            for ( int i = 0; i < menuBar.ItemCount; i++ )
            {
                MenuItem menuItem = menuBar.GetItemAt(i);
                if ( 0 == string.Compare(menuItem.Text.Replace("&", ""), "help", true) )
                {
                    menuItem.MenuItems.Add(command.GetMenuItem());
                    break;
                }
            }
        }


        /// <summary>
        /// Handles "About Demo" command.
        /// </summary>
        private void CommandAboutDemo(EditorControl editor)
        {
            AboutDialog aboutDialog = new AboutDialog();
            aboutDialog.ShowDialog();
        }

        #endregion

        #region New Command

        /// <summary>
        /// Modifies command "New", replaces its invoke handler.
        /// </summary>
        private void ModifyNewCommand()
        {
            // Here we get command instance and replace invoke handler in it.
            Command cmdNew = mEditor.Commands["New"];
            cmdNew.InvokeHandler = new CommandHandler(CommandNew);
        }


        /// <summary>
        /// Invoke handler of the "New" command.
        /// </summary>
        private void CommandNew(EditorControl editor)
        {
            if (!CheckModifiedAndSave())
                return;

            mEditor.Document = new Document();

            // We'll ask for document name on save.
            DocumentName = null;

            mEditor.Focus();
        }


        #endregion

        #region Open Command

        /// <summary>
        /// Initializes command "Send".
        /// </summary>
        private void InitOpenCommand()
        {
            // Here we get command instance and change its' invoke and enabled handlers.
            Command command = mEditor.Commands.Add("App.Open", "Open", "Opens document from the server.",
                Keys.Control | Keys.O,
                new CommandHandler(CommandOpen),
                Image.FromStream(Assembly.GetExecutingAssembly().GetManifestResourceStream("Aspose.Editor.AppPlugin.Receive.gif")),
                new ConditionHandler(IsCommandOpenEnabled));

            InsertCommandIntoToolBarAndMenu(command, true);
        }


        /// <summary>
        /// Handles "Receive" command.
        /// </summary>
        private void CommandOpen(EditorControl editor)
        {
            if ( !CheckModifiedAndSave() )
                return;

            OpenDocumentDialog dialog = new OpenDocumentDialog(this);
            dialog.ShowDialog();
        }


        private bool IsCommandOpenEnabled(EditorControl editor)
        {
            return (mEditor.ServerUrl != null) && (mEditor.ServerUrl.Length > 0);
        }


        #endregion

        #region Save Command

        /// <summary>
        /// Initializes command "Save".
        /// </summary>
        private void InitSaveCommand()
        {
            Command command = mEditor.Commands.Add("App.Save", "Save", "Saves the document to the server.",
                Keys.Control | Keys.S,
                new CommandHandler(CommandSave),
                Image.FromStream(Assembly.GetExecutingAssembly().GetManifestResourceStream("Aspose.Editor.AppPlugin.Send.gif")),
                new ConditionHandler(IsCommandSaveEnabled));

            InsertCommandIntoToolBarAndMenu(command, true);
        }


        /// <summary>
        /// Initializes command "SaveAs".
        /// </summary>
        private void InitSaveAsCommand()
        {
            Command command = mEditor.Commands.Add("App.SaveAs", "Save As...", "Prompts for the document name and saves the document to the server.",
                Keys.F12,
                new CommandHandler(CommandSaveAs),
                null,
                new ConditionHandler(IsCommandSaveEnabled));

            InsertCommandIntoToolBarAndMenu(command, false);
        }


        /// <summary>
        /// Handles "Save" command.
        /// </summary>
        private void CommandSave(EditorControl editor)
        {
            Save(false);
        }


        /// <summary>
        /// Handles "SaveAs" command.
        /// </summary>
        private void CommandSaveAs(EditorControl editor)
        {
            Save(true);
        }


        /// <summary>
        /// Implements method which evaluates value of the <see cref="Command.IsEnabled"/> property.
        /// </summary>
        private bool IsCommandSaveEnabled(EditorControl editor)
        {
            return (mEditor.Document != null) &&
                // (mDocumentName != null) &&
                (mEditor.ServerUrl != null) &&
                    // (mDocumentName.Length > 0) &&
                    (mEditor.ServerUrl.Length > 0);
        }


        /// <summary>
        /// Saves the document.
        /// </summary>
        private bool Save(bool isPrompt)
        {
            Debug.Assert(IsDocumentOpen);

            Cursor.Current = Cursors.WaitCursor;

            try
            {
                // If document name is not set (initial document), or if Save As invoked.
                if ( null == DocumentName || 0 >= DocumentName.Length || isPrompt )
                    DocumentName = GetDocumentName();

                // Name wasn't specified.
                if ( null == DocumentName )
                    return false;

                // Check upload constraints
                // Calculate the document size
                MemoryStream stream = new MemoryStream();
                mEditor.Document.Save(stream);
                long size = stream.Length;
                stream.Close();
                if(!CheckUpload(DocumentName, size))
                    return false;

                mEditor.Send(DocumentName);
            }
            catch ( Exception e )
            {
                MessageBox.Show(e.ToString(), "Demo Application");
            }
            finally
            {
                Cursor.Current = Cursors.Arrow;
            }
            return true;
        }


        /// <summary>
        /// Cheks if document with specified name and size can be uploaded.
        /// </summary>
        internal bool CheckUpload(string name, long size)
        {
            // We enforce maximum size of a single document for this demo.
            if(size > Shared.MaxDocumentSize)
            {
                MessageBox.Show(string.Format(
                    "Cannot store the document. This document is bigger than " +
                        "the limit of {0} bytes imposed by this demo project.",
                            Shared.MaxDocumentSize));
                return false;
            }

            // Get documents list on the server
            string[] list = Browse();
            if ( list != null )
            {
                bool doesDocumentNameExist = false;
                for ( int i = 0; i < list.Length; i++ )
                {
                    if(list[i] == name)
                    {
                        doesDocumentNameExist = true;
                        break;
                    }
                }

                // We enforce maximum number of documents per session for this demo.
                if ( !doesDocumentNameExist && (list.Length >= Shared.MaxDocumentsPerSession) )
                {
                    MessageBox.Show(string.Format(
                        "Cannot store the document. This demo limits the maximum number " +
                            "of documents for a single session to {0}. Please remove some documents first.",
                                Shared.MaxDocumentsPerSession));
                    return false;
                }

            }

            return true;
        }


        /// <summary>
        /// Checks that current document opened in the client control is modified and
        /// should be saved before it is closed. If document is modified then this method
        /// shows message box which asks if user wants to save document and continue, discard
        /// document and continue or cancel. If user chooses first option then document is saved.
        /// </summary>
        /// <returns>True if user wants to continue; false otherwise.</returns>
        private bool CheckModifiedAndSave()
        {
            // If there is no document or document is not modified, it is okay to continue.
            if (!IsDocumentOpen || !mEditor.Document.IsModified)
                return true;

            const string productName = "Aspose.Editor Live Demo Application";
            DialogResult dlgResult = MessageBox.Show(
                string.Format("Do you want to save changes to '{0}'?", DocumentName),
                    productName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Warning);

            switch (dlgResult)
            {
                case DialogResult.Yes:
                {
                    // Save and let the operation continue if save was successful.
                    return Save(false);
                }
                case DialogResult.No:
                    // Let the operation continue without saving the document.
                    return true;
                case DialogResult.Cancel:
                    // Signal to cancel the operation.
                    return false;
                default:
                    throw new ApplicationException("Unknown dialog result.");
            }
        }


        /// <summary>
        /// Returns <b>True</b> if document is open.
        /// </summary>
        private bool IsDocumentOpen
        {
            get { return (mEditor.Document != null); }
        }


        /// <summary>
        /// Asks user for a valid document name.
        /// </summary>
        private string GetDocumentName()
        {
            InputBox inputBox = new InputBox(DocumentName);
            string[] list = Browse();
            string result;

            while ( true )
            {
                DialogResult dialogResult = inputBox.ShowDialog();
                result = inputBox.FileName;
                if ( DialogResult.OK != dialogResult || null == result || 0 >= result.Length )
                {
                    result = null;
                    break;
                }

                if ( null != list )
                {
                    foreach (string name in list)
                    {
                        if ( 0 == string.Compare(name, result, true) )
                        {
                            dialogResult =
                                MessageBox.Show("File '" + result + "'already exists on the server. "+
                                    "Do you want to overwrite existing file ?",
                                    "Confirm Overwrite", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

                            if ( DialogResult.No == dialogResult )
                                result = null;

                            break;
                        }
                    }
                }

                if ( null != result )
                    break;
            }

            return result;
        }


        #endregion

        private void InsertCommandIntoToolBarAndMenu(Command command, bool isAddToToolbar)
        {
            if ( isAddToToolbar )
            {
                // Now we can put command to the tool bar. We'll put it into the first tool bar
                // and in second position (right after the first button).
                ToolBar toolBar = mEditor.Bars.Tools[0] as ToolBar;
                toolBar.Buttons.Insert(1, command.GetToolBarButton());
            }

            // And now we put menu item at second position in the first pop up menu.
            StandardMenuBar menu = mEditor.Bars.Menu as StandardMenuBar;
            menu.GetItemAt(0).MenuItems.Add(1, command.GetMenuItem());
        }


        /// <summary>
        /// Returns list of document names on the server.
        /// </summary>
        /// <returns>List of document names. If no documents, returns null.</returns>
        internal string[] Browse()
        {
            Cursor.Current = Cursors.WaitCursor;

            try
            {
                Stream listStream = ExecServerCommand("cmd=getlist");
                if(listStream == null)
                    return null;

                listStream.Position = 0;
                StreamReader reader = new StreamReader(listStream);
                string listString = reader.ReadToEnd();
                return listString.Substring(0, listString.Length - 1).Split(';');
            }
            catch(Exception e)
            {
                MessageBox.Show(e.ToString());
                return null;
            }
            finally
            {
                Cursor.Current = Cursors.Arrow;
            }
        }


        internal void LoadDocumentImpl(string name)
        {
            Cursor.Current = Cursors.WaitCursor;
            try
            {
                mEditor.Receive(name);
                DocumentName = name;
            }
            finally
            {
                Cursor.Current = Cursors.Arrow;
            }
        }


        /// <summary>
        /// Executes predefined server command.
        /// </summary>
        /// <param name="command">Command string.</param>
        /// <returns>Reuslt stream from the server.</returns>
        internal Stream ExecServerCommand(string command)
        {
            string url = string.Format("{0}?{1}", mEditor.ServerUrl, command);

            HttpWebRequest request;
            Stream stream = null;

            try
            {
                request = GetWebRequest(url);
                request.Method = "GET";

                stream = ParseResponse(request);
            }
            catch(Exception e)
            {
                MessageBox.Show(e.ToString());
            }

            return stream;
        }


        /// <summary>
        /// Private. Sends non-null document to URL.
        /// </summary>
        internal void UploadDocumentImpl(string filePath)
        {
            if((filePath == null) || (filePath.Length == 0))
                return;

            HttpWebRequest request;
            Stream stream = null;
            FileStream fileStream = null;

            Cursor.Current = Cursors.WaitCursor;

            try
            {
                // Open document file
                fileStream = new FileStream(filePath, FileMode.Open);
                string fileName = Path.GetFileName(filePath);

                //
                // Send document to server
                //

                request = GetWebRequest(mEditor.ServerUrl);
                request.Method = "POST";
                request.ContentLength = fileStream.Length;
                request.Headers[Shared.ParamHeader] = EncodeParam(fileName);

                stream = request.GetRequestStream();
                stream.Write(GetStreamBuffer(fileStream), 0, (int) fileStream.Length);

                // Get response and close communication.
                ParseResponse(request);
            }
            catch ( Exception e )
            {
                MessageBox.Show(e.ToString(), "Server Error", MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
            finally
            {
                if ( null != stream )
                    stream.Close();

                if ( null != fileStream )
                    fileStream.Close();

                Cursor.Current = Cursors.Arrow;
            }
        }


        /// <summary>
        /// Downloads specified document from the server.
        /// </summary>
        internal Stream DownloadDocumentImpl(string filePath, string format)
        {
            if((filePath == null) || (filePath.Length == 0))
                return null;

            HttpWebResponse response = null;
            MemoryStream result = null;
            Cursor.Current = Cursors.WaitCursor;

            try
            {
                HttpWebRequest request;
                request = GetWebRequest(mEditor.ServerUrl);
                request.Method = "GET";
                request.Headers[Shared.ParamHeader] = EncodeParam(filePath+"?"+format);

                result = ParseResponse(request);
            }
            catch ( Exception e )
            {
                MessageBox.Show(e.ToString(), "Server Error", MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
            finally
            {
                if ( null != response )
                    response.Close();

                Cursor.Current = Cursors.Arrow;
            }

            return result;
        }


        /// <summary>
        /// Encodes param value into base64 string.
        /// </summary>
        internal static string EncodeParam(string param)
        {
            byte[] bytes = Encoding.UTF8.GetBytes(param);
            return Convert.ToBase64String(bytes);
        }


        /// <summary>
        /// Decodes param value from base64 string.
        /// </summary>
        /// <param name="base64">Source string to decode.</param>
        /// <returns>Decoded string.</returns>
        internal static string DecodeParam(string base64)
        {
            byte[] bytes = Convert.FromBase64String(base64);
            return Encoding.UTF8.GetString(bytes);
        }


        /// <summary>
        /// Parses response from server.
        /// Returns the stream of the document or null if no document was sent by the server.
        /// </summary>
        internal MemoryStream ParseResponse(HttpWebRequest webRequest)
        {
            try
            {
                // This ensures web response and response stream are closed.
                using (HttpWebResponse response = (HttpWebResponse)webRequest.GetResponse())
                {
                    // In case server puts the content type but there is no content we return null.
                    // This should not happen though.
                    if ( HttpStatusCode.NoContent == response.StatusCode )
                        return null;

                    // No need to close this stream, because we are closing the whole web response.
                    // Copy data from the web response stream into a memory stream.
                    Stream responseStream = response.GetResponseStream();
                    MemoryStream stream = new MemoryStream();
                    CopyStream(responseStream, stream);
                    stream.Position = 0;

                    // In case server return content and 200 OK but no document we return null
                    // This should not happen though.
                    if (stream.Length == 0)
                        return null;

                    return stream;
                }
            }
            catch (WebException e)
            {
                if (WebExceptionStatus.ProtocolError == e.Status)
                {
                    // This is a web exception with status protocol error,
                    // it is a complete response from the server, but with an error message.
                    // Retrieve the error message and throw as exception.
                    HttpWebResponse errorResponse = (HttpWebResponse)e.Response;

                    throw new Exception(errorResponse.StatusDescription);
                }
                else
                {
                    // Error for other reason, just rethrow as is.
                    throw;
                }
            }
        }


        /// <summary>
        /// Copies from the current position in src stream till the end.
        /// Copies into the current position in dst stream.
        /// </summary>
        public static void CopyStream(Stream srcStream, Stream dstStream)
        {
            if (srcStream == null)
                throw new ArgumentNullException("srcStream");
            if (dstStream == null)
                throw new ArgumentNullException("dstStream");

            byte[] buf = new byte[4096];
            while (true)
            {
                int bytesRead = srcStream.Read(buf, 0, buf.Length);

                // Read returns 0 when reached end of stream.
                if (bytesRead == 0)
                    break;
                else
                    dstStream.Write(buf, 0, bytesRead);
            }
        }


        /// <summary>
        /// Verifies that content type name matches.
        /// </summary>
        internal static bool IsValidContentType(string contentType)
        {
            // check that content type name is valid
            return !(null == contentType || !contentType.StartsWith(Shared.ContentType + ";"));
        }


        /// <summary>
        /// Private. Preauthenticates document request.
        /// </summary>
        internal void AuthenticateRequest(HttpWebRequest request)
        {
            if ( null == Credentials )
                return;

            request.PreAuthenticate = true;
            request.Credentials = Credentials;
        }


        /// <summary>
        /// Private. If <see cref="Cookie"/> are set will put them in the specified request.
        /// </summary>
        /// <param name="request">The HttpWebRequest to put cookies into.</param>
        internal void AddCookiesToRequest(HttpWebRequest request)
        {
            if ( null != Cookie && 0 != Cookie.Length )
                request.Headers.Add("Cookie", Cookie);
        }


        /// <summary>
        /// Prepares web request.
        /// </summary>
        internal HttpWebRequest GetWebRequest(string url)
        {
            HttpWebRequest result = WebRequest.Create(url) as HttpWebRequest;
            if ( null == result )
                throw new WebException("Cannot create web request.");

            result.ContentType = Shared.ContentType;
            AuthenticateRequest(result);
            AddCookiesToRequest(result);

            return result;
        }


        /// <summary>
        /// Returns bytes of the stream. Returned buffer has same length as the stream.
        /// Stream position is not affected.
        /// </summary>
        /// <param name="stream">Stream.</param>
        /// <returns>Byte buffer.</returns>
        internal static byte[] GetStreamBuffer(Stream stream)
        {
            long streamPosition = stream.Position;
            byte[] bytes = new byte[stream.Length];
            stream.Position = 0;
            stream.Read(bytes, 0, (int)stream.Length);
            stream.Position = streamPosition;
            return bytes;
        }


        private ICredentials Credentials
        {
            get { return mEditor.Credentials; }
        }


        private string Cookie
        {
            get { return mEditor.Cookie; }
        }


        private readonly EditorControl mEditor;

        /// <summary>
        /// Gets or sets document name. This method uses reflection to update Editor.Document.Name property value.
        /// </summary>
        private string DocumentName
        {
            get { return mDocumentName; }
            set
            {
                mDocumentName = value;
                if ( null != mEditor.Document )
                {
                    // NOTE. Due to obfuscation mName field of the document will get changed to the unreadable value below.
                    // If client DLL is rebuild this value may change again.
                    FieldInfo fieldInfo = mEditor.Document.GetType().GetField("xc61ff88f1f98652d", BindingFlags.NonPublic | BindingFlags.Instance);
                    if ( null == fieldInfo )
                        MessageBox.Show("AppPlugin.DocumentName needs to be updated.");
                    else
                        fieldInfo.SetValue(mEditor.Document, "DOCUMENT: " + value);
                }
            }
        }
        private string mDocumentName;
    }
}
