//////////////////////////////////////////////////////////////////////////
// Copyright 2005-2008 Aspose Pty Ltd. All Rights Reserved.
//
// This file is part of Aspose.Editor. The source code in this file
// is only intended as a supplement to the documentation, and is provided
// "as is", without warranty of any kind, either expressed or implied.
//////////////////////////////////////////////////////////////////////////

using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Web;

namespace Aspose.Editor.Demo.Web
{
    /// <summary>
    /// This class is a server-side storage for documents used in this demo.
    ///
    /// The documents are stored as memory streams in a hashtable and the whole thing is stored in the session state.
    /// You probably wouldn't use such solution in a real-world application, but for a public live demo we don't
    /// want users to load and save to the actual file system on the server, therefore all is kept in memory.
    /// </summary>
    public class DemoDocumentStorage
    {
        /// <summary>
        /// Returns the document storage instance for the current session.
        /// </summary>
        public static DemoDocumentStorage GetInstance(HttpContext context)
        {
            DemoDocumentStorage documentStorage = context.Session[DocumentStorageName] as DemoDocumentStorage;
            if (null == documentStorage)
            {
                documentStorage = new DemoDocumentStorage(context);
                context.Session[DocumentStorageName] = documentStorage;
            }
            return documentStorage;
        }


        /// <summary>
        /// Ctor. Loads all documents from the "Documents" directory into memory.
        /// </summary>
        private DemoDocumentStorage(HttpContext context)
        {
            mDocuments = System.Collections.Specialized.CollectionsUtil.CreateCaseInsensitiveSortedList();

            // Read all files from the Document directory into the document storage.
            string docsPath = Path.Combine(context.Request.PhysicalApplicationPath, @"..\..\Common\Documents");
            DirectoryInfo directoryInfo = new DirectoryInfo(docsPath);
            foreach (FileInfo fileInfo in directoryInfo.GetFiles())
            {
                using (Stream stream = fileInfo.OpenRead())
                    CopyAndStoreDocument(fileInfo.Name, stream);
            }
        }


        /// <summary>
        /// Retrieves a document from the storage. Returns null if there is no such document.
        /// </summary>
        public MemoryStream GetDocument(string fileName)
        {
            return mDocuments[fileName] as MemoryStream;
        }


        /// <summary>
        /// Returns true if a document with the specified name exists in the storage.
        /// </summary>
        public bool Exists(string fileName)
        {
            return mDocuments.ContainsKey(fileName);
        }


        /// <summary>
        /// Gets list of documents currently in the storage.
        /// </summary>
        public string[] GetFileNames()
        {
            string[] array = new string[mDocuments.Keys.Count];
            mDocuments.Keys.CopyTo(array, 0);
            return array;
        }


        /// <summary>
        /// Saves the document into the storage. If a document with the same name already exists, overrides it.
        /// </summary>
        public void StoreDocument(string fileName, MemoryStream stream)
        {
            // We enforce maximum number of documents per session for this demo.
            if (!Exists(fileName) && (Count >= Shared.MaxDocumentsPerSession))
                throw new InvalidOperationException(string.Format(
                    "Cannot store the document. This demo limits the maximum number of documents for a single session to {0}. Please remove some documents first.",
                    Shared.MaxDocumentsPerSession));

            // We enforce maximum size of a single document for this demo.
            if (stream.Length > Shared.MaxDocumentSize)
                throw new InvalidOperationException(string.Format(
                    "Cannot store the document. This document is bigger than the limit of {0} bytes imposed by this demo project.",
                    Shared.MaxDocumentSize));

            mDocuments[fileName] = stream;
        }


        /// <summary>
        /// Copies the specified stream into a memory stream and adds it to the document storage.
        /// </summary>
        public void CopyAndStoreDocument(string fileName, Stream stream)
        {
            stream.Position = 0;
            MemoryStream docStream = new MemoryStream();
            CopyStream(stream, docStream);

            StoreDocument(fileName, docStream);
        }


        /// <summary>
        /// Deletes a document from the storage. If the document does not exist, does nothing.
        /// </summary>
        public void DeleteDocument(string fileName)
        {
            // Let's keep at least this one "built-in" demo document.
            if ( 0 == string.Compare(fileName, ControlForm.InitialDocument, true) )
                throw new Exception("This document cannot be deleted.");

            mDocuments.Remove(fileName);
        }


        /// <summary>
        /// Copies source stream to destination stream.
        /// </summary>
        private static void CopyStream(Stream srcStream, Stream dstStream)
        {
            int count;
            const int buffer_size = 4096;
            byte[] buffer = new byte[buffer_size];

            while (true)
            {
                count = srcStream.Read(buffer, 0, buffer_size);

                if (count != 0)
                    dstStream.Write(buffer, 0, count);

                if (count != buffer_size)
                    break;
            }

            dstStream.Position = 0;
        }


        /// <summary>
        /// Gets the number of documents in the storage.
        /// </summary>
        public int Count
        {
            get { return mDocuments.Count; }
        }


        private SortedList mDocuments;

        /// <summary>
        /// This is the identifier of the application document cache.
        /// </summary>
        private const string DocumentStorageName = "Aspose.DocumentCache";

    }
}