//////////////////////////////////////////////////////////////////////////
// Copyright 2005-2008 Aspose Pty Ltd. All Rights Reserved.
//
// This file is part of Aspose.Editor. The source code in this file
// is only intended as a supplement to the documentation, and is provided
// "as is", without warranty of any kind, either expressed or implied.
//////////////////////////////////////////////////////////////////////////

// NOTE
// This demo compiles for .NET 1.1. It can also be compiled as is for .NET 2.0.
// Alternatively it can be converted to the .NET 2.0 project but it will no
// longer compile in .NET 1.1.

using System;
using System.IO;
using System.Web;
using System.Text;
using System.Web.UI;
using System.Threading;
using System.Reflection;
using Aspose.Editor.Server;

namespace Aspose.Editor.Demo.Web
{
    /// <summary>
    /// Represents form on the web page which contains editor control.
    /// </summary>
    public class ControlForm : System.Web.UI.Page
    {
        /// <summary>
        /// Initializes new instance of this class.
        /// </summary>
        public ControlForm()
        {
            Load += new System.EventHandler(Page_Load);
        }


        static ControlForm()
        {
            // This code sets a license for Aspose.Editor.Server.EditorControl.
            try
            {
                Aspose.Editor.Server.License l = new Aspose.Editor.Server.License();
                l.SetLicense(@"Aspose.Editor.lic");
            }
            catch
            {
                // Ignore "license not found" exception, stay in evaluation for the demo,
                // but don't eat exceptions silently in a real-world application.
            }
        }


        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);

            EditorControl.DocumentReceived += new DocumentEventHandler(OnDocumentReceived);
            EditorControl.DocumentRequested += new DocumentEventHandler(OnDocumentRequested);
        }


        /// <summary>
        /// Instance of the editor server control running on the page.
        /// </summary>
        protected Aspose.Editor.Server.EditorControl EditorControl;

        /// <summary>
        /// This method is called by the ASP.NET when page is loading.
        /// </summary>
        protected void Page_Load(object sender, EventArgs e)
        {
            try
            {
                // PostPack is True if page containing editor control is submitted, e.g. ASP button is clicked,
                // or if page is opened for the first time.
                // Client editor control doesn't submit page, but initiates new requests.
                if ( !IsPostBack )
                {
                    // Process commands (app demo only).
                    // This method doesn't return if request is recognized and processed.
                    ProcessCommands();

                    // Process document upload (app demo only).
                    // This method doesn't return if request is recognized and processed.
                    ProccessDocumentUpload();

                    // Process document download (app demo only).
                    // This method doesn't return if request is recognized and processed.
                    ProcessDocumentDownload();

                    // Check for and process document requests from the client.
                    // This method doesn't return if request is recognized and processed.
                    EditorControl.OnPageLoad();

                    // Verify that client is capable of hosting client editor control.
                    new ClientVersionInformation(Request).VerifyClientIsValid(this);

                    // If the client has requested the page for the first time, specify the document stream to preload.
                    EditorControl.PreloadStream = DocumentStorage.GetDocument(InitialDocument);

                    // Here is an alternative way to load a document into the client control on open.
                    // If you specify PreloadUrl and PreloadParam, the client control will make a callback
                    // to the server to get a document. Then, the server can decide which document to return,
                    // it can read existing or create new document and send it.
                    // EditorControl.PreloadUrl = Request.Url.ToString();
                    // EditorControl.PreloadParam = InitialDocument;
                }
            }
            catch ( ThreadAbortException )
            {
                // Calls to End, Redirect, and Transfer raise a ThreadAbortException when the current response ends prematurely.
                throw;
            }
            catch (Exception ex)
            {
                Response.StatusCode = 500;
                Response.StatusDescription = ex.Message;
            }
        }


        /// <summary>
        /// This demo is simple and keeps all documents loaded in this cache object in the session state.
        /// </summary>
        public DemoDocumentStorage DocumentStorage
        {
            get { return DemoDocumentStorage.GetInstance(Context); }
        }


        /// <summary>
        /// Checks for and processes document upload requests from the client.
        /// This method doesn't return if request is recognized and processed.
        /// </summary>
        private void ProccessDocumentUpload()
        {
            if ((Request.ContentLength > 0) && (Request.ContentType == Shared.ContentType) && (Request.HttpMethod == "POST") )
            {
                // Get document name from header.
                string docName = DecodeParam(Request.Headers[Shared.ParamHeader]);
                DocumentStorage.CopyAndStoreDocument(docName, Request.InputStream);

                CloseResponse();
            }
        }


        /// <summary>
        /// Checks for and processes document download request from the client.
        /// This method doesn't return if request is recognized and processed.
        /// </summary>
        private void ProcessDocumentDownload()
        {
            string base64 = Request.Headers[Shared.ParamHeader];
            if ( null == base64 || base64.Length <= 0 )
                return;

            string docName = DecodeParam(base64);
            string format = string.Empty;
            if ( 0 <= docName.IndexOf('?') )
            {
                string[] temp = docName.Split('?');
                docName = temp[0];
                if ( 1 < temp.Length )
                    format = temp[1];
            }

            MemoryStream stream = DocumentStorage.GetDocument(docName);
            if (null == stream)
                throw new Exception("Document not found.");

            Response.ContentType = Shared.ContentType;

            stream.Position = 0;
            byte[] buffer = new byte[stream.Length];
            stream.Read(buffer, 0, (int)stream.Length);

            // TODO: server doesn't support conversions of the documents. Use Aspose.Words for this task.
            switch (format)
            {
                case "doc":
                case "dot":
                case "docx":
                case "docm":
                case "mhtml":
                case "odt":
                case "html":
                case "rtf":
                case "xml":
                    break;
            }

            Response.BinaryWrite(buffer);

            CloseResponse();
        }


        /// <summary>
        /// Encodes param value into base64 string.
        /// </summary>
        public static string EncodeParam(string param)
        {
            byte[] bytes = Encoding.UTF8.GetBytes(param);
            return Convert.ToBase64String(bytes);
        }


        /// <summary>
        /// Decodes param value from base64 string.
        /// </summary>
        /// <param name="base64">Source string to decode.</param>
        /// <returns>Decoded string.</returns>
        public static string DecodeParam(string base64)
        {
            byte[] bytes = Convert.FromBase64String(base64);
            return Encoding.UTF8.GetString(bytes);
        }


        /// <summary>
        /// This is the name of the document in the <see cref="DemoDocumentStorage"/> which is
        /// preloaded to the client editor control running in the browser.
        /// </summary>
        public const string InitialDocument = "welcome.xml";

        /// <summary>
        /// Checks for and processes application commands.
        /// This method doesn't return if request is recognized and processed.
        /// </summary>
        private void ProcessCommands()
        {
            string clientCommand = Request.QueryString["cmd"];
            switch (clientCommand)
            {
                case "getlist":
                    BrowseCommand();
                    break;
                case "delete":
                    DeleteCommand();
                    break;
            }
        }


        /// <summary>
        /// Implements "delete" client command.
        /// </summary>
        private void DeleteCommand()
        {
            string file = Request.QueryString["file"];
            DocumentStorage.DeleteDocument(file);
            CloseResponse();
        }


        /// <summary>
        /// Implements "browse" client command.
        /// </summary>
        private void BrowseCommand()
        {
            string[] files = DocumentStorage.GetFileNames();
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < files.Length; i++)
                sb.AppendFormat("{0};", files[i]);

            Response.ContentType = "docList";
            Response.Output.Write(sb.ToString());

            CloseResponse();
        }


        /// <summary>
        /// Closes response and terminates current thread.
        /// </summary>
        private void CloseResponse()
        {
            Response.StatusCode = 200; // HttpStatusCode.OK
            Response.Flush();
            Response.End();
        }


        /// <summary>
        /// This method is called when the client requests a document from the server.
        /// </summary>
        private void OnDocumentRequested(object sender, DocumentEventArgs e)
        {
            // Get a document from the storage. In this demo we get the document from the document cache.
            // In a real-word application this could be from anywhere (file system, database, sharepoint etc).
            string fileName = e.Param;
            Stream docStream = DocumentStorage.GetDocument(fileName);
            if (docStream != null)
            {
                // Pass the document to the client via the event argument.
                docStream.Position = 0;
                e.SetDocument(docStream);

                // We don't close the stream here, because it is a memory stream of a document in the cache,
                // but in a real-world application, you would probably need to close the stream here.
            }
        }


        /// <summary>
        /// This method is called when the server receives a document from the client.
        /// </summary>
        private void OnDocumentReceived(object sender, DocumentEventArgs e)
        {
            string fileName = e.Param;

            DocumentFormat documentFormat;

            switch ( Path.GetExtension(fileName).ToLower() )
            {
                case ".doc":
                    documentFormat = DocumentFormat.Doc;
                    break;
                case ".docx":
                    documentFormat = DocumentFormat.Docx;
                    break;
                case ".docm":
                    documentFormat = DocumentFormat.Docm;
                    break;
//                case ".dotx":
//                    documentFormat = DocumentFormat.Dotx;
//                    break;
//                case ".dotm":
//                    documentFormat = DocumentFormat.Dotm;
//                    break;
                case ".html": case ".htm":
                    documentFormat = DocumentFormat.Html;
                    break;
                case ".rtf":
                    documentFormat = DocumentFormat.Rtf;
                    break;
                case ".pdfxml":
                    documentFormat = DocumentFormat.AsposePdf;
                    break;
                case ".txt":
                    documentFormat = DocumentFormat.Text;
                    break;
                case ".odt":
                    documentFormat = DocumentFormat.Odt;
                    break;
                default:
                    documentFormat = DocumentFormat.WordML;
                    break;
            }

            // Retrieve the document into a memory stream. You can request the document in any format.
            MemoryStream docStream = e.GetDocument(documentFormat);

            // Store the document. In this demo we store the document to the document cache.
            // In a real-word application it is up to you where to store the document,
            // it could be anywhere (file system, database, sharepoint etc).
            DocumentStorage.StoreDocument(fileName, docStream);
        }
    }

    [Flags]
    public enum ClientCompatibility : int
    {
        Browser = 16,
        Runtime = 8,
        Platform = 4,
        Script = 2,
        Cookies = 1,
        None = 0,
    }

    /// <summary>
    /// Stores browser version information.
    /// </summary>
    public class ClientVersionInformation
    {
        /// <summary>
        /// Reads version information from the first request.
        /// </summary>
        /// <remarks>Method shall be called only on initial page load.</remarks>
        public ClientVersionInformation(HttpRequest request)
        {
            if (null == request)
                return;

            Platform = request.Browser.Platform;
            BrowserName = request.Browser.Browser;
            BrowserVersion = request.Browser.Version;
            IsCookiesSupported = request.Browser.Cookies;
            IsScriptSupported = (request.Browser.EcmaScriptVersion.Major >= 1);

            Version[] clrVersions = request.Browser.GetClrVersions();
            ClrVersions = new string[clrVersions.Length];
            StringBuilder stringBuilder = new StringBuilder();
            for (int i = 0; i < clrVersions.Length; i++)
            {
                ClrVersions[i] = clrVersions[i].ToString();
                stringBuilder.AppendFormat("{0}{1}", (0 < i ? "; " : string.Empty), clrVersions[i]);
            }
            ClrVersion = stringBuilder.ToString();
        }


        /// <summary>
        /// Platform which runs client browser.
        /// </summary>
        public readonly string Platform;          // WinXP, not Vista

        /// <summary>
        /// Browser which has sent this request.
        /// </summary>
        public readonly string BrowserName;       // IE, not others

        /// <summary>
        /// Client browser version.
        /// </summary>
        public readonly string BrowserVersion;    // 5.5 and above

        /// <summary>
        /// <b>True</b> if cookies are enabled.
        /// </summary>
        public readonly bool IsCookiesSupported;  // True

        /// <summary>
        /// <b>True</b> if jscript is supported.
        /// </summary>
        public readonly bool IsScriptSupported;   // True

        /// <summary>
        /// List of CLR version installed on the client.
        /// </summary>
        public readonly string[] ClrVersions;     // 1.1.4322

        /// <summary>
        /// CLR versions separated with "; ".
        /// This is used to display them in the dialog.
        /// </summary>
        public readonly string ClrVersion;

        public override string ToString()
        {
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.Append(Platform);
            stringBuilder.Append("; ");
            stringBuilder.Append(BrowserName);
            stringBuilder.Append(" ");
            stringBuilder.Append(BrowserVersion);
            stringBuilder.Append("; (");
            stringBuilder.Append(ClrVersion);
            stringBuilder.Append("); Script=");
            stringBuilder.Append(IsScriptSupported);
            stringBuilder.Append("; Cookies=");
            stringBuilder.Append(IsCookiesSupported);
            return stringBuilder.ToString();
        }


        /// <summary>
        /// Verifies that client is OK to run. If not then this methods
        /// posts response and terminates thread.
        /// </summary>
        /// <param name="page"></param>
        public void VerifyClientIsValid(Page page)
        {
            if (IsValidBrowser && IsValidPlatform && IsValidRuntime && IsCookiesSupported && IsScriptSupported)
                return;

            Assembly assembly = Assembly.GetExecutingAssembly();
            string resName = assembly.GetName().Name + ".ClientIncompatible.html";
            using (Stream stream = assembly.GetManifestResourceStream(resName))
            {
                StreamReader streamReader = new StreamReader(stream);
                StringBuilder sb = new StringBuilder(streamReader.ReadToEnd());
                sb.Replace("_0_", Platform);
                sb.Replace("_1_", BrowserName + " " + BrowserVersion);
                sb.Replace("_2_", ClrVersion);
                sb.Replace("_3_", IsScriptSupported ? "Enabled" : "Disabled");
                sb.Replace("_4_", IsCookiesSupported ? "Enabled" : "Disabled");
                sb.Replace("_5_", IsValidPlatform ? "ok" : "ko");
                sb.Replace("_6_", IsValidBrowser ? "ok" : "ko");
                sb.Replace("_7_", IsValidRuntime ? "ok" : "ko");
                sb.Replace("_8_", IsScriptSupported ? "ok" : "ko");
                sb.Replace("_9_", IsCookiesSupported ? "ok" : "ko");

                page.Response.Write(sb.ToString());
                page.Response.Flush();
                page.Response.End();
            }
        }


        /// <summary>
        /// Returns client compatibility rating.
        /// </summary>
        public int ClientCompatibility
        {
            get
            {
                ClientCompatibility compat = Web.ClientCompatibility.None;

                if (IsValidBrowser)
                    compat |= Web.ClientCompatibility.Browser;

                if (IsValidRuntime)
                    compat |= Web.ClientCompatibility.Runtime;

                if (IsValidPlatform)
                    compat |= Web.ClientCompatibility.Platform;

                if (IsCookiesSupported)
                    compat |= Web.ClientCompatibility.Cookies;

                return (int)compat;
            }
        }


        /// <summary>
        /// Returns <b>True</b> if client platform is Windows XP.
        /// </summary>
        public bool IsValidPlatform
        {
            get
            {
                switch (Platform)
                {
                    case "WinXP":
                    case "WinNT":
                    case "Win2000":
                        return true;
                    default:
                        return false;
                }
            }
        }


        /// <summary>
        /// Returns <b>True</b> if client browser is Internet Explorer 5.5 or higher.
        /// </summary>
        public bool IsValidBrowser
        {
            get
            {
                try
                {
                    return (("IE" == BrowserName) &&
                            (int.Parse(BrowserVersion.Substring(0, BrowserVersion.IndexOf('.'))) >= 5));
                }
                catch
                {
                    return false;
                }
            }
        }


        /// <summary>
        /// Returns <b>True</b> if client has 1.1 runtime installed.
        /// </summary>
        public bool IsValidRuntime
        {
            get
            {
                return (0 <= ClrVersion.IndexOf("2.0.")) ||
                       (0 <= ClrVersion.IndexOf("1.1."));
            }
        }


        /// <summary>
        /// Returns <b>True</b> if client has JScript 1.2 installed.
        /// </summary>
        public bool IsValidScript
        {
            get
            {
                return (0 <= ClrVersion.IndexOf("1.1."));
            }
        }
    }
}
